function accs = demo_filter_methods(method, nbins, varargin) % k_folds, svm_kernel, C, gamma)
%
% nbins default should be around 10
% accepted methods: snr, welch, ranksum, kmeans, *feast_methods*
%
% To run supply:
%   - data_file_ind (60 for multiclass miRNA),
%   - classifier (knn, svm, ...),
%   - k_folds (5, 10, ...),
%   - optional: svm params (c, gamma, svm_kernel)
%
% e.g. demo_filter_methods('snr', 10, 'data_file_ind', 60, 'classifier', 'knn', 'k_folds', 5, 'c', 1.0, 'gamma', 0.005, 'svm_kernel', 2);

p = parse_params(varargin{:});
p

rng('shuffle');

[data_set data_labels] = fetch_data(p.data_file_ind);
[ntrain, D] = size(data_set);

% XXX 
% SUBS_SIZE = min(D, 200);
SUBS_SIZE = D; % 100;

subset_inds = cell(D, 1);

feast_methods = {'mrmr' 'mim' 'mifs' 'cmim' 'jmi' 'disr' 'cife' 'icap'...
                 'condred' 'cmi' 'relief' 'fcbf' 'betagamma'};

method_name = method;
if strcmp(method, 'snr')
    ranks = mysnr(data_set, data_labels);
    ranks = max(ranks);
    [~, order] = sort(ranks, 'descend');
    order = order(1:SUBS_SIZE);
    subset_inds = cell_of_subsets(order);
elseif strcmp(method, 'welch')
    tic
    ranks = welch_test(data_set, data_labels, 4000);
    toc
    % load('welch_12_28-Nov-2014_23-46-13.mat');
    % load('welch_13_29-Nov-2014_00-24-09.mat');
    % load('welch_10_29-Nov-2014_01-10-31.mat');
    % load('welch_4_29-Nov-2014_00-45-18.mat');
    filename = strcat('welch_', num2str(ind), '_', strrep(strrep(datestr(clock), ' ', '_'), ':', '-'), '.mat');
    save(filename, 'ranks');
    ranks = min(ranks);
    [~, order] = sort(ranks, 'descend'); % NOTE was: 'ascend'
    order = order(1:SUBS_SIZE);
    subset_inds = cell_of_subsets(order);
elseif strcmp(method, 'ranksum')
    ranks = ranksum_test(data_set, data_labels);
    ranks = min(ranks);
    [~, order] = sort(ranks, 'ascend');
    order = order(1:SUBS_SIZE);
    subset_inds = cell_of_subsets(order);
elseif strcmp(method, 'kmeans')
    tic
    subset_inds = k_means_rank(data_set, data_labels, SUBS_SIZE);
    toc
elseif strmatch(method, feast_methods)
    x = normalize_data(data_set, nbins);
    subset_inds = cell(SUBS_SIZE, 1);
    for i = 2:SUBS_SIZE
        subset_inds{i} = feast(method_name,i,x,data_labels);
    end
    % fprintf(1, ' %d %.5f\n', length(inds), accuracy);
    method_name = strcat(method_name, '_nbins=', num2str(nbins)); % XXX This is bogus as a method name.
else
    fprintf(1, 'Unknown method!\n');
    return
end

%--------Create log directory
if ~exist(p.logdir, 'dir')
    mkdir(p.logdir);
end

fname = strcat(p.logdir, '/',...
               'data=', num2str(p.data_file_ind),...
               '_method=', num2str(method_name),...
               '_cls=', p.classifier,...
               '_svmkernel=', num2str(p.svm_kernel),...
               '_kfolds=', num2str(p.k_folds),...
               '_C=', num2str(p.c),...
               '_gamma=', num2str(p.gamma));

[~, hostname] = system('hostname');
fname = strcat(fname, '_', strrep(strrep(p.start_time, ' ', '_'), ':', '-'),...
                      '_', num2str(p.randi));
fid = fopen(fname , 'a');

if exist('ranks', 'var')
    fprintf(fid, '# p-values / rank values\n');
    fprintf(fid, '%s\n', mat2str(ranks));
    fprintf(fid, '# Genes ordered from the most signif. one\n');
    foo = 1:D;
    fprintf(fid, '%s\n', mat2str(foo(order)));
end

accs = zeros(SUBS_SIZE-1, 2);
for i = 2:SUBS_SIZE
    inds = subset_inds{i};
    % inds
    % size(data_set)
    % XX = data_set(:, inds);
    [fit score] = score_fset_multiclass(...
        inds, 'data_file_ind', p.data_file_ind, 'k_folds', p.k_folds,...
        'classifier', p.classifier, 'c', p.c, 'gamma', p.gamma,...
        'svm_kernel', p.svm_kernel);
    accs(i,1) = score;
    accs(i,2) = i;
    fprintf(fid, ' %d %.5f\n', i, score);
end

[val, pos] = max(accs(:, 1));
fprintf(fid, '# Best one\n%.5f\n%d\n', val, accs(pos,2));

if exist('order', 'var')
    fprintf(fid, '%s\n', mat2str(order(1:(accs(pos,2)))));
end

fprintf(fid, '\n%s\n', mat2str(subset_inds{i}));
fprintf(fid, '\n%s\n', mat2str(subset_inds{SUBS_SIZE}));
fclose(fid);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ret = cell_of_subsets(indices)
% Returns a cell array of subsets of increasing size, i.e. for [5, 2, 3, 1, 5] it returns
% ret{2} = [5, 2] ; ret{3} = [5, 2, 3] ; ret{4} = [5, 2, 3, 1] and so on

ret = cell(length(indices), 1);

for i = 2:length(indices)
    ret{i} = indices(1:i);
    % if i + 10 < length(indices)
    %     ret{i} = indices(i-1:i+9); % XXX XXX XXX
    % end
end

end

