function [fitness score] = inner_loop(inds, repr, X, train_labels, Z, test_labels, param)

inds = unique(inds);

nfeatures = length(inds);
[ninstances ~] = size(X);

if nfeatures == 0
    fitness = 0.0;
    score = 0.0;
    return;
end

if isnan(param.gamma)
    libsvm_params = ['-q -t ' num2str(param.svm_kernel) ' -c ' num2str(param.c)];
else
    libsvm_params = ['-q -t ' num2str(param.svm_kernel) ' -c ' num2str(param.c) ' -g ' num2str(param.gamma)];
end

%-------------- Built-in classifiers
% http://www.mathworks.com/help/stats/choose-a-classifier.html
% https://www.mathworks.com/help/stats/crossval.html
if ~strcmp(param.classifier, 'svm')
    XX = X(:, inds);
    YY = train_labels;

    cp = cvpartition(YY,'k',param.k_folds);
    if strcmp(param.classifier, 'knn')
        % https://www.mathworks.com/help/stats/classificationknn.crossval.html
        model = ClassificationKNN.fit(XX,YY, 'NumNeighbors', param.knn_k);
        cvmdl = crossval(model, 'KFold', param.k_folds);
        cvmdlloss = kfoldLoss(cvmdl);
        % fitcknn(XX,YY);
    elseif strcmp(param.classifier, 'adaboost')
        % http://www.mathworks.com/help/stats/ensemble-methods.html
        model = fitensemble(XX,YY,'AdaBoostM2',100,'Tree');
        cvmdl = crossval(model, 'KFold', param.k_folds);
        cvmdlloss = kfoldLoss(cvmdl);
    elseif strcmp(param.classifier, 'totalboost50')
        % http://www.mathworks.com/help/stats/ensemble-methods.html
        model = fitensemble(XX,YY,'TotalBoost',50,'Tree');
        cvmdl = crossval(model, 'KFold', param.k_folds);
        cvmdlloss = kfoldLoss(cvmdl);
    elseif strcmp(param.classifier, 'totalboost100')
        % http://www.mathworks.com/help/stats/ensemble-methods.html
        model = fitensemble(XX,YY,'TotalBoost',100,'Tree');
        cvmdl = crossval(model, 'KFold', param.k_folds);
        cvmdlloss = kfoldLoss(cvmdl);
    elseif strcmp(param.classifier, 'randomforest')
        % http://www.mathworks.com/help/stats/treebagger.html
        % model = TreeBagger(200,XX,YY,'Method','classification');
        pred = @(xtrain, ytrain,xtest)...
               (predict(TreeBagger(50,xtrain,ytrain), xtest));
        cvmdlloss = crossval('mcr',XX ,YY,'predfun', pred ,'partition',cp);
    elseif strcmp(param.classifier, 'randomforestbosque')
        % https://www.mathworks.com/matlabcentral/fileexchange/31036-random-forest
        cvmdlloss = crossval('mcr',XX ,YY,'predfun', @bosque_pred ,'partition',cp);
    elseif strcmp(param.classifier, 'logistic')
        % http://www.mathworks.com/help/stats/mnrfit.html
        model = mnrfit(XX,YY);
        cvmdl = crossval(model, 'KFold', param.k_folds);
        cvmdlloss = kfoldLoss(cvmdl);
    elseif strcmp(param.classifier, 'naivebayes')
        % http://www.mathworks.com/help/stats/classificationnaivebayes-class.html
        % model = NaiveBayes.fit(XX,YY);
        pred = @(xtrain, ytrain,xtest)...
               (predict(NaiveBayes.fit(xtrain,ytrain), xtest));
        cvmdlloss = crossval('mcr',XX ,YY,'predfun', pred ,'partition',cp);
    elseif strcmp(param.classifier, 'clstree')
        model = ClassificationTree.fit(XX, YY);
        cvmdl = crossval(model, 'KFold', param.k_folds);
        cvmdlloss = kfoldLoss(cvmdl);
    else
        exception = MException('InnerLoop', 'Unknown classifier type');
        throw(exception);
    end

    % Normalize to 0-100.
    score = 100.0 - (100.0 * cvmdlloss);

%-------------- Old code for AdaBoost
elseif param.svm_kernel == 10

    XX = X(:, inds);
    ZZ = Z(:, inds);

    ens = fitensemble(XX,train_labels,'AdaBoostM2', 100,'Tree');
    labels = predict(ens, ZZ);

    accuracy = sum(labels == test_labels) / length(test_labels) * 100;
    fitness = round(100000 * accuracy) / 100000;
    fitness = accuracy + ((1/nfeatures) / 10000000000);
    
%------------ The rest is for LIBSVM
elseif param.k_folds == 0

    XX = X(:, inds);
    ZZ = Z(:, inds);

    [~, model] = evalc('svmtrain(train_labels, XX, ''-t 0'')');
    [~, ll, accuracy, est] = evalc('svmpredict(test_labels, ZZ, model)');

    accuracy = accuracy(1);

elseif param.svm_metric == 'f'

    XX = X(:, inds);
    % Set labels to {-1,+1}.
    YY = train_labels;
    YY(find(YY == min(YY))) = -1;
    YY(find(YY == max(YY))) = 1;

    score = do_binary_cross_validation(YY, XX, libsvm_params, param.k_folds);
    % Normalize to 0-100.
    score = 100.0 * score;

else % param.svm_metric == 'acc'

    XX = X(:, inds);
    [~, accuracy] = evalc(['svmtrain(train_labels, XX, ''' libsvm_params ' -v ' num2str(param.k_folds) ''')']);
    score = accuracy;

    % if isnan(param.gamma)
    %     [~, accuracy] = evalc(['svmtrain(train_labels, XX, ''-t ' num2str(param.svm_kernel) ' -v ' num2str(param.k_folds) ' -c ' num2str(param.C) ''')']);
    % else
    %     [~, accuracy] = evalc(['svmtrain(train_labels, XX, ''-t ' num2str(param.svm_kernel) ' -v ' num2str(param.k_folds) ' -c ' num2str(param.C) ' -g ' num2str(param.gamma) ''')']);
    % end
end

% fitness = round(100000 * accuracy) / 100000;
% fitness = score + ((1/nfeatures) / 10000000000);

%
% NOTE When updating this code remember to update save_log.m inverse
%      function recovering score from fitness
%
fitness = score - (max(0, nfeatures - param.t) * param.mirna_alpha);

end
