function obj = ReprManifold_init(D, manifold, manifold_name)
%
% This representation "walks" over a (t-)SNE manifold.
% Individual is encoded as:
%   [--weights--][--dim1--]...[--dimN--] 
% Each segment has length D <= n_features.
% There are N+1 segments, where N is the dimensionality of the manifold.
%

if ~exist('manifold_name', 'var')
    manifold_name = 'UNK';
end

obj = struct();

% private data members
[nfeats,mdim] = size(manifold);
obj.D = D;
obj.mdim = mdim; % Manifold dimensionality.

% Rescale manifold dimensions to [0,1] so the vectors would be consistent.
for d = 1:mdim
    col = manifold(1:end, d);
    w = max(col) - min(col);
    col = col - min(col); % Shift to 0
    col = col + abs(w / nfeats); % with a little margin before the first feature
    m_ = max(col) + (2*abs(w / nfeats));
    col = col / m_; % Now col should be within [0,1] with small margins around 0 and 1
    manifold(1:end, d) = col;
end
obj.manifold = manifold;

% public data members
obj.name = strcat('manifold', num2str(obj.D), manifold_name);
obj.log_name = '_MANIFOLD_';
obj.len = obj.D*(obj.mdim+1);
obj.de_FVr_minbound = zeros(1, obj.len);
obj.de_FVr_maxbound = ones(1, obj.len);

% public methods
obj.rand_pop = @ReprManifold_rand_pop;
obj.feature_num = @ReprManifold_feature_num;
obj.feature_list = @ReprManifold_feature_list;
obj.gene_list = @ReprManifold_gene_list;
obj.empty_indiv = @ReprManifold_empty_indiv;
end

function X = ReprManifold_rand_pop(obj, N)
X = rand(N, obj.len);
end

function flist = ReprManifold_feature_list(obj, indiv)
% Select indices which exceed the threshold.
ind_list = find(indiv(1:obj.D) >= 0.1);
% Select coordinates of those scaled to [0-1].
indiv_ = reshape(indiv, [obj.D, obj.mdim+1]);
coords = indiv_(ind_list,2:end);
% Find their 1-NNs in the manifold.
dist = mypdist2(obj.manifold, coords);
[~,argmin] = min(dist);
flist = unique(argmin);
end

function n = ReprManifold_feature_num(obj, indiv)
n = length(obj.feature_list(obj, indiv));
% ind_list = unique(find(indiv(1:obj.D) >= 0.1));
% n = length(ind_list);
% assert(n <= obj.D);
end

function glist = ReprManifold_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end
