function obj = ReprRandManifold3_init(D, manifold, manifold_name)
%
% This representation "walks" over a (t-)SNE manifold.
% Individual is encoded as:
%   [--weights--][--dim1--]...[--dimN--] 
% Each segment has length D <= n_features.
%
% NOTE It differs from Manifold2 in the way the population is initialized.
% NOTE In the above, it is drawn with uniform on the embedding.
% NOTE This is wrong as it assigns very high probabilities to furthest points.
% NOTE Here, initial points are picked exactly where the datapoints are located.

if ~exist('manifold_name', 'var')
    manifold_name = 'UNK';
end

obj = struct();

% private data members
[nfeats,mdim] = size(manifold);
obj.nfeats = nfeats;
obj.D = D;
obj.mdim = mdim; % Manifold dimensionality.

% Rescale manifold dimensions to [0,1] so the vectors would be consistent.
for d = 1:mdim
    col = manifold(1:end, d);
    w = max(col) - min(col);
    col = col - min(col); % Shift to 0
    col = col + abs(w / nfeats); % with a little margin before the first feature
    m_ = max(col) + (2*abs(w / nfeats));
    col = col / m_; % Now col should be within [0,1] with small margins around 0 and 1
    manifold(1:end, d) = col;
end
obj.manifold = manifold;

% public data members
obj.name = strcat('manifold3', num2str(obj.D), manifold_name);
obj.log_name = '_MANIFOLD3_';
obj.len = obj.D*obj.mdim;
obj.de_FVr_minbound = zeros(1, obj.len);
obj.de_FVr_maxbound = ones(1, obj.len);

% public methods
obj.rand_pop = @ReprRandManifold3_rand_pop;
obj.feature_num = @ReprRandManifold3_feature_num;
obj.feature_list = @ReprRandManifold3_feature_list;
obj.gene_list = @ReprRandManifold3_gene_list;
obj.empty_indiv = @ReprRandManifold3_empty_indiv;
end

function X = ReprRandManifold3_rand_pop(obj, N)
X = zeros(N, obj.len);
for i = 1:N
    % Draw D indices for a single individual
    inds = randi(obj.nfeats, obj.D, 1);
    X(i, :) = reshape(obj.manifold(inds, :), [1, obj.D * obj.mdim]);
end

end

function flist = ReprRandManifold3_feature_list(obj, indiv)
coords = reshape(indiv, [obj.D, obj.mdim]);
% Find their 1-NNs in the manifold.
dist = mypdist2(obj.manifold, coords);
[~,argmin] = min(dist);
flist = unique(argmin);
end

function n = ReprRandManifold3_feature_num(obj, indiv)
n = length(obj.feature_list(obj, indiv));
% ind_list = unique(find(indiv(1:obj.D) >= 0.1));
% n = length(ind_list);
% assert(n <= obj.D);
end

function glist = ReprRandManifold3_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end
