function obj = ReprRandManifoldMPWeights_init(D, manifold, manifold_name, cm)
%
% This representation "walks" over a (t-)SNE manifold.
% Individual is encoded as:
%   [--weights--][--dim1--]...[--dimN--] 
% Each segment has length D <= n_features.
%

if ~exist('manifold_name', 'var')
    manifold_name = 'UNK';
end

obj = struct();

% private data members
[nfeats,mdim] = size(manifold);
obj.D = D;
obj.mdim = mdim; % Manifold dimensionality.
obj.cm = cm;

% Rescale manifold dimensions to [0,1] so the vectors would be consistent.
for d = 1:mdim
    col = manifold(1:end, d);
    w = max(col) - min(col);
    col = col - min(col); % Shift to 0
    col = col + abs(w / nfeats); % with a little margin before the first feature
    m_ = max(col) + (2*abs(w / nfeats));
    col = col / m_; % Now col should be within [0,1] with small margins around 0 and 1
    manifold(1:end, d) = col;
end
obj.manifold = manifold;

% public data members
obj.name = strcat('randmanifoldmp_weights', num2str(obj.D), manifold_name);
obj.log_name = '_RANDMANIFOLDMPWeights_';
obj.len_with_weights = obj.D * (obj.mdim + 1);
obj.len = obj.len_with_weights;
obj.len_indiv = obj.D * obj.mdim;
obj.de_FVr_minbound = zeros(1, obj.len_with_weights);
obj.de_FVr_maxbound = ones(1, obj.len_with_weights);

% public methods
obj.rand_pop = @ReprRandManifold2_rand_pop;
obj.feature_num = @ReprRandManifold2_feature_num;
obj.feature_list = @ReprRandManifold2_feature_list;
obj.gene_list = @ReprRandManifold2_gene_list;
obj.empty_indiv = @ReprRandManifold2_empty_indiv;
end

function X = ReprRandManifold2_rand_pop(obj, N)
X = rand(N, obj.len_with_weights);

[nfeats,mdim] = size(obj.manifold);
inds = randi([1, nfeats], N, obj.D);
XX = zeros(N, obj.len_indiv);
for i = (1:N)
    XX(i, :) = reshape(obj.manifold(inds(i, :), :), 1, obj.len_indiv);
end
X(:, 1:obj.len_indiv) = XX;
end

function flist = ReprRandManifold2_feature_list(obj, indiv)
coords = reshape(indiv(1:obj.len_indiv), [obj.D, obj.mdim]);
weights = indiv(obj.len_indiv + 1 : obj.len_with_weights);
coords = coords(weights >= 0.25, :);
% Find their 1-NNs in the manifold.
dist = mypdist2(obj.manifold, coords);
[~,argmin] = min(dist);
flist = unique(argmin);
% Translate based on CM
flist = unique(obj.cm(flist));
end

function n = ReprRandManifold2_feature_num(obj, indiv)
n = length(obj.feature_list(obj, indiv));
% ind_list = unique(find(indiv(1:obj.D) >= 0.1));
% n = length(ind_list);
% assert(n <= obj.D);
end

function glist = ReprRandManifold2_gene_list(obj, indiv)
glist = obj.feature_list(obj, indiv);
end
